/*
 * Preferences.cs: Preferences window.
 * Copyright (c) 2004 Daniel Serpell
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using Gtk;
using System;

public class Preferences {
    
    [Glade.Widget]
    private Dialog dlgPreferences;
    
    [Glade.Widget]
    private Button closeButton;
    
    [Glade.Widget]
    private Button helpButton;
    
    [Glade.Widget]
    private SpinButton sbZoom;
    
    [Glade.Widget]
    private CheckButton cbZoomDiag;

    [Glade.Widget]
    private Combo cbProjType;
    
    [Glade.Widget]
    private VBox vbAdvancedProj;
    
    [Glade.Widget]
    private HScale hsXAxisRound;
    
    [Glade.Widget]
    private HScale hsYAxisRound;
    
    [Glade.Widget]
    private Combo cbSetView;
    
    [Glade.Widget]
    private VBox bvAdvancedView;
    
    [Glade.Widget]
    private SpinButton sbAzimuth;
    
    [Glade.Widget]
    private SpinButton sbElevation;
    
    [Glade.Widget]
    private SpinButton sbRotation;
    
    // Refference to the view
    private ViewData view;
    
    // True if updating the view values from code.
    private bool updatingView;
    
    // True if updating the controls values from code.
    private bool updatingControls;
    
    private void setViewDirection()
    {
        double az,el,rt;
        az = sbAzimuth.Value * (System.Math.PI / 180);
        el = sbElevation.Value * (System.Math.PI / 180);
        rt = sbRotation.Value * (System.Math.PI / 180);
        updatingView = true;
        view.setAzimutElevRot(az,el,rt);
        updatingView = false;
    }
    
    private void getViewDirection()
    {
        double az,el,rt;
        view.getAzimutElevRot(out az,out el,out rt);
        updatingControls = true;
        sbAzimuth.Value = az * (180 / System.Math.PI);
        sbElevation.Value = el * (180 / System.Math.PI);
        sbRotation.Value = rt * (180 / System.Math.PI);
        cbSetView.Entry.Text = "Select One";
        updatingControls = false;
    }
    
    private void getProjectionValues()
    {
        updatingControls = true;
        hsXAxisRound.Value = view.XAxisCurvature;
        hsYAxisRound.Value = view.YAxisCurvature;
        updatingControls = false;
    }
    
    private void getProjectionWord()
    {
        updatingControls = true;
        if( (view.XAxisCurvature == 0) && (view.YAxisCurvature == 0) )
        {
            cbProjType.Entry.Text = "Rectangular";
        }
        else if ( (view.XAxisCurvature == 1) && (view.YAxisCurvature == 0) )
        {
            cbProjType.Entry.Text = "Cylindrical";
        }
        else if ( (view.XAxisCurvature == 1) && (view.YAxisCurvature == 1) )
        {
            cbProjType.Entry.Text = "Spherical";
        }
        else
            cbProjType.Entry.Text = "Custom";
        updatingControls = false;
    }
    
    private void FillControls()
    {
        updatingControls = true;
        sbZoom.Value = System.Math.Log(view.Zoom,1.25);
        cbZoomDiag.Active = view.UseDiag;
        updatingControls = false;
        
        getProjectionValues();
        getProjectionWord();
        getViewDirection();        
    }
    
    public Preferences(ViewData vw)
    {
        view = vw;
        Glade.XML xml = new Glade.XML ("pastemage.glade", "Preferences", null);
        
        xml.Autoconnect(this);
        
        // Combo-boxes are not-editable!
        cbProjType.Entry.Editable = false;
        cbSetView.Entry.Editable = false;
        
        FillControls();
        view.NotifyUpdate += OnUpdateView;
    }
    
    void OnUpdateView(object sender, EventArgs args)
    {
        if( updatingView ) return;
        FillControls();
    }
    
    void OnSbZoomValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        updatingView = true;
        view.Zoom = System.Math.Pow(1.25,sbZoom.Value);
        updatingView = false;
    }
    
    void onCbZoomDiagToggled(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        updatingView = true;
        view.UseDiag = cbZoomDiag.Active;
        updatingView = false;
    }
    
    void OnCbProjTypeChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        string s = cbProjType.Entry.Text;
        updatingView = true;
        if( s == "Rectangular" )
        {
            view.XAxisCurvature = 0;
            view.YAxisCurvature = 0;
        }
        else if( s == "Cylindrical" )
        {
            view.XAxisCurvature = 1;
            view.YAxisCurvature = 0;
        }
        else if( s == "Spherical" )
        {
            view.XAxisCurvature = 1;
            view.YAxisCurvature = 1;
        }
        updatingView = false;
        getProjectionValues();
    }    
    
    void OnHsXAxisRoundValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        updatingView = true;
        view.XAxisCurvature = hsXAxisRound.Value;
        updatingView = false;
        getProjectionWord();
    }
    
    void OnHsYAxisRoundValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        updatingView = true;
        view.YAxisCurvature = hsYAxisRound.Value;
        updatingView = false;
        getProjectionWord();
    }
    
    void OnCbSetViewChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        updatingView = true;
        string s = cbSetView.Entry.Text;
        if( s == "North" )
            view.goNorth();
        else if( s == "South" )
            view.goSouth();
        else if( s == "North-East" )
            view.goNorthEast();
        else if( s == "South-East" )
            view.goSouthEast();
        else if( s == "North-West" )
            view.goNorthWest();
        else if( s == "South-West" )
            view.goSouthWest();
        else if( s == "East" )
            view.goEast();
        else if( s == "West" )
            view.goWest();
        else if( s == "Down" )
            view.goDown();
        else if( s == "Up" )
            view.goUp();
        updatingView = false;
        
        getViewDirection();
    }
    
    void OnSbAzimuthValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        setViewDirection();
    }
    
    void OnSbElevationValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        setViewDirection();
    }
    
    void OnSbRotationValueChanged(object sender, EventArgs args)
    {
        if( updatingControls ) return;
        setViewDirection();
    }
    
}
