/*
 * TileCache.cs: Class to mantain a cache of the tiled-view and projection.
 * Copyright (c) 2004 Daniel Serpell
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using Vector3D;

public class TileCache {

    public struct ImageInter {
        public double ax, bx, cx, dx;
        public double ay, by, cy, dy;
        public bool visible;
    }
    public struct Tile {
        public int x,y;  // Tile position on screen
        public v3d dir;  // View direction.
        public ImageInter[] imgInterArray; // Array with image interpolation data
    }

    private bool validView;
    private bool validImage;
    public Tile[,] tiles;
    private int sx,sy;
    private ViewData view;
    private double zoom, cx, cy, xac, yac;
    private v3d dir,dx,dy;
    
    public TileCache() {
        validView = false;
        validImage = false;
    }
    
    public int getTilesX() { return sx; }
    public int getTilesY() { return sy; }
    
    public void invalidateView() {
        validView = false;
        validImage = false;
    }
    
    public void invalidateImage() {
        validImage = false;
    }
    
    public bool ValidView() {
        return validView;
    }
    
    public bool ValidImage() {
        return validImage;
    }
    
    public v3d viewVector(double x, double y)
    {
        // The view multiplicands:
        double md = 1,
               mx = zoom * (x - cx),
               my = zoom * (y - cy);
        
        if( xac > 1e-20 )
        {
            mx = System.Math.Sin( xac * mx ) / xac;
            md = (System.Math.Cos( xac * mx ) - 1) / xac + 1;
        }
        if( yac > 1e-20 )
            my = System.Math.Sin( yac * my ) / yac;
        
        return md * dir + mx * dx + my * dy;
    }
    
    public v3d viewDirection(double x, double y)
    {
        return zoom* (x - cx) * dx + zoom * (y - cy) * dy;
    }
    
    public void updateViewCache(ViewData newView)
    {
        // The view-cache stores, for each tile in the image,
        // the vector with the view-direction of the top-left
        // pixel.
        view = newView;
        
        // Copy view parameters to internal variables
	if( view.sx < view.sy )
            zoom = view.Zoom / view.sx;
        else
            zoom = view.Zoom / view.sy;
        cx = view.sx * 0.5;
        cy = view.sy * 0.5;
        dir = view.Dir;
        dx = view.Dx;
        dy = view.Dy;
        xac = view.XAxisCurvature;
        yac = view.YAxisCurvature;
        
        // Tile-cache size
        sx = 1 + ((view.sx - 1) / view.TileSize);
        sy = 1 + ((view.sy - 1) / view.TileSize);
        
        tiles = new Tile[sx+1,sy+1];
        for(int tx=0; tx<=sx; tx++)
        {
            for(int ty=0; ty<=sy; ty++)
            {
                int x = tx * view.TileSize;
                int y = ty * view.TileSize;
                tiles[tx,ty].x = x;
                tiles[tx,ty].y = y;
                tiles[tx,ty].dir = viewVector(x,y);
            }
        }
        validView = true;
        validImage = false;
    }
    
    // Fill's the image-projection cache
    public void updateImageCache(System.Collections.ArrayList images)
    {
        int n = images.Count;
        double iN = 1.0 / view.TileSize;
        double iN2 = iN*iN;
        for(int tx = 0; tx<sx; tx++)
        {
            for(int ty = 0; ty<sy; ty++)
            {
                // Four direction vectors
                v3d vw00 = tiles[tx,ty].dir;
                v3d vw01 = tiles[tx,ty+1].dir;
                v3d vw10 = tiles[tx+1,ty].dir;
                v3d vw11 = tiles[tx+1,ty+1].dir;
                
                // Fill's each tile image-projection
                tiles[tx,ty].imgInterArray = new ImageInter[n];
                for(int i=0;i<n;i++)
                {
                    tiles[tx,ty].imgInterArray[i].visible = false;
                    ImageWindow.imageInfo im = (ImageWindow.imageInfo)images[i];
                    double a00 = v3d.dot(im.N,vw00);
	            if( a00 <= 0 )
	                continue;
                    double a01 = v3d.dot(im.N,vw01);
	            if( a01 <= 0 )
	                continue;
                    double a10 = v3d.dot(im.N,vw10);
	            if( a10 <= 0 )
	                continue;
                    double a11 = v3d.dot(im.N,vw11);
	            if( a11 <= 0 )
	                continue;
		    a00 = im.scale / a00;
                    double u00 = a00*v3d.dot(im.U,vw00) + im.sx/2;
                    double v00 = a00*v3d.dot(im.V,vw00) + im.sy/2;
		    a01 = im.scale / a01;
                    double u01 = a01*v3d.dot(im.U,vw01) + im.sx/2;
                    double v01 = a01*v3d.dot(im.V,vw01) + im.sy/2;
		    a10 = im.scale / a10;
                    double u10 = a10*v3d.dot(im.U,vw10) + im.sx/2;
                    double v10 = a10*v3d.dot(im.V,vw10) + im.sy/2;
		    a11 = im.scale / a11;
                    double u11 = a11*v3d.dot(im.U,vw11) + im.sx/2;
                    double v11 = a11*v3d.dot(im.V,vw11) + im.sy/2;
                    tiles[tx,ty].imgInterArray[i].visible = true;
                    tiles[tx,ty].imgInterArray[i].ax = u00;
                    tiles[tx,ty].imgInterArray[i].bx = (u10-u00)*iN;
                    tiles[tx,ty].imgInterArray[i].cx = (u01-u00)*iN;
                    tiles[tx,ty].imgInterArray[i].dx = (u11-u10-u01+u00)*iN2;
                    tiles[tx,ty].imgInterArray[i].ay = v00;
                    tiles[tx,ty].imgInterArray[i].by = (v10-v00)*iN;
                    tiles[tx,ty].imgInterArray[i].cy = (v01-v00)*iN;
                    tiles[tx,ty].imgInterArray[i].dy = (v11-v10-v01+v00)*iN2;
                }
            }
        }
        validImage = true;
    }
    
}
